@@ -618,7 +618,7 @@ chat_command_join_cb (EmpathyDispatchOperation *dispatch,
 	if (error != NULL) {
 		DEBUG ("Error: %s", error->message);
 		empathy_chat_view_append_event (chat->view,
-			_("Failed to join chatroom"));
+			_("Failed to join chat room"));
 	}
 }
 
@@ -695,13 +695,24 @@ static void
 chat_command_join (EmpathyChat *chat,
 		   GStrv        strv)
 {
+	guint i = 0;
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	TpConnection *connection;
 
-	connection = empathy_tp_chat_get_connection (priv->tp_chat);
-	empathy_dispatcher_join_muc (connection, strv[1],
-				     chat_command_join_cb,
-				     chat);
+	GStrv rooms = g_strsplit_set (strv[1], ", ", -1);
+
+	while (rooms[i] != NULL) {
+		/* ignore empty strings */
+		if (EMP_STR_EMPTY (rooms[i])) {
+			TpConnection *connection;
+
+			connection = empathy_tp_chat_get_connection (priv->tp_chat);
+			empathy_dispatcher_join_muc (connection, rooms[i],
+						     chat_command_join_cb,
+						     chat);
+		}
+		i++;
+	}
+	g_strfreev (rooms);
 }
 
 static void
@@ -799,36 +810,36 @@ typedef struct {
 
 static ChatCommandItem commands[] = {
 	{"clear", 1, 1, chat_command_clear,
-	 N_("/clear, clear all messages from the current conversation")},
+	 N_("/clear: clear all messages from the current conversation")},
 
 	{"topic", 2, 2, chat_command_topic,
-	 N_("/topic <topic>, set the topic of the current conversation")},
+	 N_("/topic <topic>: set the topic of the current conversation")},
 
 	{"join", 2, 2, chat_command_join,
-	 N_("/join <chatroom id>, join a new chatroom")},
+	 N_("/join <chat room ID>: join a new chat room")},
 
 	{"j", 2, 2, chat_command_join,
-	 N_("/j <chatroom id>, join a new chatroom")},
+	 N_("/j <chat room ID>: join a new chat room")},
 
 	{"query", 2, 3, chat_command_query,
-	 N_("/query <contact id> [<message>], open a private chat")},
+	 N_("/query <contact ID> [<message>]: open a private chat")},
 
 	{"msg", 3, 3, chat_command_msg,
-	 N_("/msg <contact id> <message>, open a private chat")},
+	 N_("/msg <contact ID> <message>: open a private chat")},
 
 	{"nick", 2, 2, chat_command_nick,
-	 N_("/nick <nickname>, change your nickname on current server")},
+	 N_("/nick <nickname>: change your nickname on the current server")},
 
 	{"me", 2, 2, chat_command_me,
-	 N_("/me <message>, send an ACTION message to the current conversation")},
+	 N_("/me <message>: send an ACTION message to the current conversation")},
 
 	{"say", 2, 2, chat_command_say,
-	 N_("/say <message>, send <message> to the current conversation. "
+	 N_("/say <message>: send <message> to the current conversation. "
 	    "This is used to send a message starting with a '/'. For example: "
-	    "\"/say /join is used to join a new chatroom\"")},
+	    "\"/say /join is used to join a new chat room\"")},
 
 	{"help", 1, 2, chat_command_help,
-	 N_("/help [<command>], show all supported commands. "
+	 N_("/help [<command>]: show all supported commands. "
 	    "If <command> is defined, show its usage.")},
 };
 
@@ -988,7 +999,7 @@ chat_send (EmpathyChat  *chat,
 
 		if (!second_slash) {
 			empathy_chat_view_append_event (chat->view,
-				_("Unknown command, see /help for the available"
+				_("Unknown command; see /help for the available"
 				  " commands"));
 			return;
 		}
@@ -1270,6 +1281,13 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
 	}
 }
 
+static gboolean
+empathy_isspace_cb (gunichar c,
+		 gpointer data)
+{
+	return g_unichar_isspace (c);
+}
+
 static gboolean
 chat_input_key_press_event_cb (GtkWidget   *widget,
 			       GdkEventKey *event,
@@ -1364,7 +1382,9 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 
 		/* Get the start of the nick to complete. */
 		gtk_text_buffer_get_iter_at_mark (buffer, &start, gtk_text_buffer_get_insert (buffer));
-		gtk_text_iter_backward_word_start (&start);
+		if (gtk_text_iter_backward_find_char (&start, &empathy_isspace_cb, NULL, NULL)) {
+			gtk_text_iter_set_offset (&start, gtk_text_iter_get_offset (&start) + 1);
+		}
 		is_start_of_buffer = gtk_text_iter_is_start (&start);
 
 		list = empathy_contact_list_get_members (EMPATHY_CONTACT_LIST (priv->tp_chat));
@@ -1381,6 +1401,8 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 			guint        len;
 			const gchar *text;
 			gchar       *complete_char = NULL;
+			GString     *message = NULL;
+			GList       *l;
 
 			gtk_text_buffer_delete (buffer, &start, &current);
 
@@ -1396,6 +1418,18 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 				text = empathy_contact_get_name (completed_list->data);
 			} else {
 				text = completed;
+
+				/* Print all hits to the scrollback view, so the
+				 * user knows what possibilities he has.
+				 * Fixes #599779
+				 * */
+				 message = g_string_new ("");
+				 for (l = completed_list; l != NULL; l = l->next) {
+					g_string_append (message, empathy_contact_get_name (l->data));
+					g_string_append (message, " - ");
+				 }
+				 empathy_chat_view_append_event (chat->view, message->str);
+				 g_string_free (message, TRUE);
 			}
 
 			gtk_text_buffer_insert_at_cursor (buffer, text, strlen (text));
