@@ -37,6 +37,7 @@
 
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-connection-managers.h>
+#include <libempathy/empathy-connectivity.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include <libempathy-gtk/empathy-protocol-chooser.h>
@@ -50,6 +51,7 @@
 #include "empathy-accounts-dialog.h"
 #include "empathy-import-dialog.h"
 #include "empathy-import-utils.h"
+#include "ephy-spinner.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_ACCOUNT
 #include <libempathy/empathy-debug.h>
@@ -60,23 +62,26 @@
 /* The primary text of the dialog shown to the user when he is about to lose
  * unsaved changes */
 #define PENDING_CHANGES_QUESTION_PRIMARY_TEXT \
-  _("There are unsaved modification regarding your %s account.")
+  _("There are unsaved modifications to your %s account.")
 /* The primary text of the dialog shown to the user when he is about to lose
  * an unsaved new account */
 #define UNSAVED_NEW_ACCOUNT_QUESTION_PRIMARY_TEXT \
   _("Your new account has not been saved yet.")
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyAccountsDialog)
-G_DEFINE_TYPE (EmpathyAccountsDialog, empathy_accounts_dialog, G_TYPE_OBJECT);
+G_DEFINE_TYPE (EmpathyAccountsDialog, empathy_accounts_dialog, GTK_TYPE_DIALOG);
 
 static EmpathyAccountsDialog *dialog_singleton = NULL;
 
 typedef struct {
-  GtkWidget *window;
-
   GtkWidget *alignment_settings;
+  GtkWidget *alignment_infobar;
 
   GtkWidget *vbox_details;
+  GtkWidget *infobar;
+  GtkWidget *label_status;
+  GtkWidget *image_status;
+  GtkWidget *throbber;
   GtkWidget *frame_no_protocol;
 
   GtkWidget *treeview;
@@ -84,13 +89,8 @@ typedef struct {
   GtkWidget *button_add;
   GtkWidget *button_import;
 
-  GtkWidget *frame_new_account;
   GtkWidget *combobox_protocol;
-  GtkWidget *hbox_type;
-  GtkWidget *button_create;
-  GtkWidget *button_back;
-  GtkWidget *radiobutton_reuse;
-  GtkWidget *radiobutton_register;
+  GtkWidget *hbox_protocol;
 
   GtkWidget *image_type;
   GtkWidget *label_name;
@@ -125,7 +125,7 @@ typedef struct {
   gboolean force_change_row;
   GtkTreeRowReference *destination_row;
 
-
+  gboolean dispose_has_run;
 } EmpathyAccountsDialogPriv;
 
 enum {
@@ -153,6 +153,26 @@ static void accounts_dialog_model_select_first (EmpathyAccountsDialog *dialog);
 static void accounts_dialog_update_settings (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings);
 
+static void accounts_dialog_add (EmpathyAccountsDialog *dialog,
+    EmpathyAccountSettings *settings);
+
+static void accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
+    EmpathyAccountSettings *settings);
+
+static void accounts_dialog_connection_changed_cb (TpAccount *account,
+    guint old_status,
+    guint current,
+    guint reason,
+    gchar *dbus_error_name,
+    GHashTable *details,
+    EmpathyAccountsDialog *dialog);
+
+static void accounts_dialog_presence_changed_cb (TpAccount *account,
+    guint presence,
+    gchar *status,
+    gchar *status_message,
+    EmpathyAccountsDialog *dialog);
+
 static void
 accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
     const gchar *display_name)
@@ -166,6 +186,145 @@ accounts_dialog_update_name_label (EmpathyAccountsDialog *dialog,
   g_free (text);
 }
 
+static void
+accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
+    TpAccount *account)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  const gchar               *message;
+  gchar                     *status_message = NULL;
+  guint                     status;
+  guint                     reason;
+  guint                     presence;
+  EmpathyConnectivity       *connectivity;
+  GtkTreeView               *view;
+  GtkTreeModel              *model;
+  GtkTreeSelection          *selection;
+  GtkTreeIter               iter;
+  TpAccount                 *selected_account;
+  gboolean                  account_enabled;
+  gboolean                  creating_account;
+
+  view = GTK_TREE_VIEW (priv->treeview);
+  selection = gtk_tree_view_get_selection (view);
+
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return;
+
+  gtk_tree_model_get (model, &iter,
+      COL_ACCOUNT_POINTER, &selected_account, -1);
+
+  /* do not update the infobar when the account is not selected */
+  if (account != selected_account)
+    return;
+
+  if (account != NULL)
+    {
+      status = tp_account_get_connection_status (account, &reason);
+      presence = tp_account_get_current_presence (account, NULL, &status_message);
+      account_enabled = tp_account_is_enabled (account);
+      creating_account = FALSE;
+    }
+  else
+    {
+      status = TP_CONNECTION_STATUS_DISCONNECTED;
+      presence = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
+      account_enabled = FALSE;
+      creating_account = TRUE;
+    }
+
+  gtk_image_set_from_icon_name (GTK_IMAGE (priv->image_status),
+      empathy_icon_name_for_presence (presence), GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+  if (account_enabled)
+    {
+      switch (status)
+        {
+          case TP_CONNECTION_STATUS_CONNECTING:
+            message = _("Connecting…");
+            gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                GTK_MESSAGE_INFO);
+
+            ephy_spinner_start (EPHY_SPINNER (priv->throbber));
+            gtk_widget_show (priv->throbber);
+            gtk_widget_hide (priv->image_status);
+            break;
+          case TP_CONNECTION_STATUS_CONNECTED:
+            if (g_strcmp0 (status_message, "") == 0)
+              {
+                message = g_strdup_printf ("%s",
+                    empathy_presence_get_default_message (presence));
+              }
+            else
+              {
+                message = g_strdup_printf ("%s — %s",
+                    empathy_presence_get_default_message (presence),
+                    status_message);
+              }
+            gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                GTK_MESSAGE_INFO);
+
+            gtk_widget_show (priv->image_status);
+            gtk_widget_hide (priv->throbber);
+            break;
+          case TP_CONNECTION_STATUS_DISCONNECTED:
+            message = g_strdup_printf (_("Disconnected — %s"),
+                empathy_status_reason_get_default_message (reason));
+
+            if (reason == TP_CONNECTION_STATUS_REASON_REQUESTED)
+              {
+                message = g_strdup_printf (_("Offline — %s"),
+                    empathy_status_reason_get_default_message (reason));
+                gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                    GTK_MESSAGE_WARNING);
+              }
+            else
+              {
+                gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                    GTK_MESSAGE_ERROR);
+              }
+
+            connectivity = empathy_connectivity_dup_singleton ();
+            if (!empathy_connectivity_is_online (connectivity))
+               message = _("Offline — No Network Connection");
+
+            g_object_unref (connectivity);
+            ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+            gtk_widget_show (priv->image_status);
+            gtk_widget_hide (priv->throbber);
+            break;
+          default:
+            message = _("Unknown Status");
+            gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+                GTK_MESSAGE_WARNING);
+
+            ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+            gtk_widget_hide (priv->image_status);
+            gtk_widget_hide (priv->throbber);
+        }
+    }
+  else
+    {
+      message = _("Offline — Account Disabled");
+
+      gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
+          GTK_MESSAGE_WARNING);
+      ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+      gtk_widget_show (priv->image_status);
+      gtk_widget_hide (priv->throbber);
+    }
+
+  gtk_label_set_text (GTK_LABEL (priv->label_status), message);
+  gtk_widget_show (priv->label_status);
+
+  if (!creating_account)
+    gtk_widget_show (priv->infobar);
+  else
+    gtk_widget_hide (priv->infobar);
+
+  g_free (status_message);
+}
+
 static void
 empathy_account_dialog_widget_cancelled_cb (
     EmpathyAccountWidget *widget_object,
@@ -204,17 +363,23 @@ empathy_account_dialog_widget_cancelled_cb (
       g_object_unref (account);
     }
 
+  gtk_widget_set_sensitive (priv->treeview, TRUE);
+  gtk_widget_set_sensitive (priv->button_add, TRUE);
+  gtk_widget_set_sensitive (priv->button_import, TRUE);
+
   if (settings != NULL)
     g_object_unref (settings);
 }
 
 static void
 empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
+    TpAccount *account,
     EmpathyAccountsDialog *dialog)
 {
   gchar *display_name;
   EmpathyAccountSettings *settings =
       accounts_dialog_model_get_selected_settings (dialog);
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
   display_name = empathy_account_widget_get_default_display_name (
       widget_object);
@@ -225,6 +390,17 @@ empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
   g_free (display_name);
 
   accounts_dialog_update_settings (dialog, settings);
+  accounts_dialog_update_status_infobar (dialog,
+      empathy_account_settings_get_account (settings));
+
+  gtk_widget_set_sensitive (priv->treeview, TRUE);
+  gtk_widget_set_sensitive (priv->button_add, TRUE);
+  gtk_widget_set_sensitive (priv->button_import, TRUE);
+
+  empathy_signal_connect_weak (account, "status-changed",
+      G_CALLBACK (accounts_dialog_connection_changed_cb), G_OBJECT (dialog));
+  empathy_signal_connect_weak (account, "presence-changed",
+      G_CALLBACK (accounts_dialog_presence_changed_cb), G_OBJECT (dialog));
 
   if (settings)
     g_object_unref (settings);
@@ -235,7 +411,8 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings)
 {
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
-  gchar *icon_name;
+  gchar                     *icon_name;
+  TpAccount                 *account;
 
   priv->setting_widget_object =
       empathy_account_widget_new_for_protocol (settings, FALSE);
@@ -243,8 +420,6 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
   priv->settings_widget =
       empathy_account_widget_get_widget (priv->setting_widget_object);
 
-  priv->settings_widget =
-      empathy_account_widget_get_widget (priv->setting_widget_object);
   g_signal_connect (priv->setting_widget_object, "account-created",
         G_CALLBACK (empathy_account_dialog_account_created_cb), dialog);
   g_signal_connect (priv->setting_widget_object, "cancelled",
@@ -272,6 +447,9 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
 
   accounts_dialog_update_name_label (dialog,
       empathy_account_settings_get_display_name (settings));
+
+  account = empathy_account_settings_get_account (settings);
+  accounts_dialog_update_status_infobar (dialog, account);
 }
 
 static void
@@ -327,71 +505,81 @@ accounts_dialog_has_pending_change (EmpathyAccountsDialog *dialog,
 }
 
 static void
-accounts_dialog_protocol_changed_cb (GtkWidget *widget,
-    EmpathyAccountsDialog *dialog)
+accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
 {
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  EmpathyAccountSettings *settings;
+  gchar *str;
+  const gchar *display_name;
   TpConnectionManager *cm;
   TpConnectionManagerProtocol *proto;
   gboolean is_gtalk;
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
   cm = empathy_protocol_chooser_dup_selected (
       EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk);
-
   if (cm == NULL)
     return;
 
-  if (proto == NULL)
-    {
-      g_object_unref (cm);
-      return;
-    }
+  display_name = empathy_protocol_name_to_display_name (
+      is_gtalk ? "gtalk" : proto->name);
 
-#ifndef HAVE_MOBLIN
-  if (tp_connection_manager_protocol_can_register (proto) && !is_gtalk)
-    {
-      gtk_widget_show (priv->radiobutton_register);
-      gtk_widget_show (priv->radiobutton_reuse);
-    }
-  else
-    {
-      gtk_widget_hide (priv->radiobutton_register);
-      gtk_widget_hide (priv->radiobutton_reuse);
-    }
-#else
-  gtk_widget_hide (priv->radiobutton_register);
-  gtk_widget_hide (priv->radiobutton_reuse);
-#endif
+  if (display_name == NULL)
+    display_name = proto->name;
 
+  /* Create account */
+  /* To translator: %s is the name of the protocol, such as "Google Talk" or
+   * "Yahoo!"
+   */
+  str = g_strdup_printf (_("New %s account"), display_name);
+  settings = empathy_account_settings_new (cm->name, proto->name, str);
+
+  g_free (str);
+
+  if (is_gtalk)
+    empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
+        NULL, NULL);
+
+  accounts_dialog_add (dialog, settings);
+  accounts_dialog_model_set_selected (dialog, settings);
+
+  gtk_widget_show_all (priv->hbox_protocol);
+
+  g_object_unref (settings);
   g_object_unref (cm);
 }
 
 static void
-accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
+accounts_dialog_protocol_changed_cb (GtkWidget *widget,
+    EmpathyAccountsDialog *dialog)
 {
-  GtkTreeView *view;
-  GtkTreeModel *model;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gboolean creating;
 
-  view = GTK_TREE_VIEW (priv->treeview);
-  model = gtk_tree_view_get_model (view);
+  /* The "changed" signal is fired during the initiation of the
+   * EmpathyProtocolChooser while populating the widget. Such signals should
+   * be ignored so we check if we are actually creating a new account. */
+  if (priv->setting_widget_object == NULL)
+    return;
 
-  gtk_widget_set_sensitive (priv->button_add, FALSE);
-  gtk_widget_hide (priv->vbox_details);
-  gtk_widget_hide (priv->frame_no_protocol);
-  gtk_widget_show (priv->frame_new_account);
+  g_object_get (priv->setting_widget_object,
+      "creating-account", &creating, NULL);
+  if (!creating)
+    return;
 
-  /* If we have no account, no need of a back button */
-  if (gtk_tree_model_iter_n_children (model, NULL) > 0)
-    gtk_widget_show (priv->button_back);
-  else
-    gtk_widget_hide (priv->button_back);
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
 
-  accounts_dialog_protocol_changed_cb (priv->radiobutton_register, dialog);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->radiobutton_reuse),
-      TRUE);
-  gtk_combo_box_set_active (GTK_COMBO_BOX (priv->combobox_protocol), 0);
-  gtk_widget_grab_focus (priv->combobox_protocol);
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return;
+
+  /* We are creating a new widget to replace the current one, don't ask
+   * confirmation to the user. */
+  priv->force_change_row = TRUE;
+  gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+
+  accounts_dialog_setup_ui_to_add_account (dialog);
 }
 
 static void
@@ -406,9 +594,8 @@ accounts_dialog_show_question_dialog (EmpathyAccountsDialog *dialog,
   va_list button_args;
   GtkWidget *message_dialog;
   const gchar *button_text;
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
-  message_dialog = gtk_message_dialog_new (GTK_WINDOW (priv->window),
+  message_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
       GTK_MESSAGE_QUESTION,
       GTK_BUTTONS_NONE,
@@ -474,6 +661,7 @@ accounts_dialog_button_add_clicked_cb (GtkWidget *button,
     EmpathyAccountsDialog *dialog)
 {
   TpAccount *account = NULL;
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
   if (accounts_dialog_has_pending_change (dialog, &account))
     {
@@ -493,6 +681,9 @@ accounts_dialog_button_add_clicked_cb (GtkWidget *button,
   else
     {
       accounts_dialog_setup_ui_to_add_account (dialog);
+      gtk_widget_set_sensitive (priv->treeview, FALSE);
+      gtk_widget_set_sensitive (priv->button_add, FALSE);
+      gtk_widget_set_sensitive (priv->button_import, FALSE);
     }
 }
 
@@ -541,7 +732,6 @@ accounts_dialog_update_settings (EmpathyAccountsDialog *dialog,
 
       /* No account and no profile, warn the user */
       gtk_widget_hide (priv->vbox_details);
-      gtk_widget_hide (priv->frame_new_account);
       gtk_widget_show (priv->frame_no_protocol);
       gtk_widget_set_sensitive (priv->button_add, FALSE);
       return;
@@ -549,10 +739,9 @@ accounts_dialog_update_settings (EmpathyAccountsDialog *dialog,
 
   /* We have an account selected, destroy old settings and create a new
    * one for the account selected */
-  gtk_widget_hide (priv->frame_new_account);
   gtk_widget_hide (priv->frame_no_protocol);
   gtk_widget_show (priv->vbox_details);
-  gtk_widget_set_sensitive (priv->button_add, TRUE);
+  gtk_widget_hide (priv->hbox_protocol);
 
   if (priv->settings_widget)
     {
@@ -1128,6 +1317,9 @@ accounts_dialog_connection_changed_cb (TpAccount *account,
   gboolean      found;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
+  /* Update the status-infobar in the details view */
+  accounts_dialog_update_status_infobar (dialog, account);
+
   /* Update the status in the model */
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
@@ -1158,6 +1350,17 @@ accounts_dialog_connection_changed_cb (TpAccount *account,
         dialog);
 }
 
+static void
+accounts_dialog_presence_changed_cb (TpAccount *account,
+    guint presence,
+    gchar *status,
+    gchar *status_message,
+    EmpathyAccountsDialog *dialog)
+{
+  /* Update the status-infobar in the details view */
+  accounts_dialog_update_status_infobar (dialog, account);
+}
+
 static void
 accounts_dialog_account_display_name_changed_cb (TpAccount *account,
   GParamSpec *pspec,
@@ -1224,7 +1427,7 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   accounts_dialog_connection_changed_cb (account,
       0,
       status,
-      TP_CONNECTION_STATUS_DISCONNECTED,
+      TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED,
       NULL,
       NULL,
       dialog);
@@ -1290,6 +1493,10 @@ enable_or_disable_account (EmpathyAccountsDialog *dialog,
   /* Update the status in the model */
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
+  /* Update the status-infobar in the details view when disabling */
+  if (!enabled)
+    accounts_dialog_update_status_infobar (dialog, account);
+
   DEBUG ("Account %s is now %s",
       tp_account_get_display_name (account),
       enabled ? "enabled" : "disabled");
@@ -1311,87 +1518,13 @@ accounts_dialog_account_enabled_cb (TpAccountManager *manager,
   enable_or_disable_account (dialog, account, TRUE);
 }
 
-static void
-accounts_dialog_button_create_clicked_cb (GtkWidget *button,
-    EmpathyAccountsDialog *dialog)
-{
-  EmpathyAccountSettings *settings;
-  gchar *str;
-  const gchar *display_name;
-  TpConnectionManager *cm;
-  TpConnectionManagerProtocol *proto;
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
-  gboolean is_gtalk;
-
-  cm = empathy_protocol_chooser_dup_selected (
-      EMPATHY_PROTOCOL_CHOOSER (priv->combobox_protocol), &proto, &is_gtalk);
-
-  display_name = empathy_protocol_name_to_display_name (
-      is_gtalk ? "gtalk" : proto->name);
-
-  if (display_name == NULL)
-    display_name = proto->name;
-
-  /* Create account */
-  /* To translator: %s is the name of the protocol, such as "Google Talk" or
-   * "Yahoo!"
-   */
-  str = g_strdup_printf (_("New %s account"), display_name);
-  settings = empathy_account_settings_new (cm->name, proto->name, str);
-
-  g_free (str);
-
-#ifndef HAVE_MOBLIN
-  if (tp_connection_manager_protocol_can_register (proto))
-    {
-      gboolean active;
-
-      active = gtk_toggle_button_get_active
-        (GTK_TOGGLE_BUTTON (priv->radiobutton_register));
-      if (active)
-        empathy_account_settings_set_boolean (settings, "register", TRUE);
-    }
-#endif
-
-  if (is_gtalk)
-    empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
-        NULL, NULL);
-
-  accounts_dialog_add (dialog, settings);
-  accounts_dialog_model_set_selected (dialog, settings);
-
-  g_object_unref (settings);
-  g_object_unref (cm);
-}
-
-static void
-accounts_dialog_button_back_clicked_cb (GtkWidget *button,
-    EmpathyAccountsDialog *dialog)
-{
-  EmpathyAccountSettings *settings;
-
-  settings = accounts_dialog_model_get_selected_settings (dialog);
-  accounts_dialog_update_settings (dialog, settings);
-
-  if (settings)
-    g_object_unref (settings);
-}
-
-static void
-accounts_dialog_button_help_clicked_cb (GtkWidget *button,
-    EmpathyAccountsDialog *dialog)
-{
-  empathy_url_show (button, "ghelp:empathy?accounts-window");
-}
-
 static void
 accounts_dialog_button_import_clicked_cb (GtkWidget *button,
     EmpathyAccountsDialog *dialog)
 {
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   GtkWidget *import_dialog;
 
-  import_dialog = empathy_import_dialog_new (GTK_WINDOW (priv->window),
+  import_dialog = empathy_import_dialog_new (GTK_WINDOW (dialog),
       FALSE);
   gtk_widget_show (import_dialog);
 }
@@ -1409,33 +1542,6 @@ accounts_dialog_close_response_cb (GtkDialog *message_dialog,
     gtk_widget_destroy (account_dialog);
 }
 
-static void
-accounts_dialog_response_cb (GtkWidget *widget,
-    gint response,
-    EmpathyAccountsDialog *dialog)
-{
-  TpAccount *account = NULL;
-
-  if (accounts_dialog_has_pending_change (dialog, &account))
-    {
-      gchar *question_dialog_primary_text = get_dialog_primary_text (account);
-
-      accounts_dialog_show_question_dialog (dialog,
-          question_dialog_primary_text,
-          _("You are about to close the window, which will discard\n"
-              "your changes. Are you sure you want to proceed?"),
-          G_CALLBACK (accounts_dialog_close_response_cb),
-          widget,
-          GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
-          GTK_STOCK_DISCARD, GTK_RESPONSE_YES, NULL);
-
-      g_free (question_dialog_primary_text);
-    }
-  else if (response == GTK_RESPONSE_CLOSE ||
-           response == GTK_RESPONSE_DELETE_EVENT)
-    gtk_widget_destroy (widget);
-}
-
 static gboolean
 accounts_dialog_delete_event_cb (GtkWidget *widget,
     GdkEvent *event,
@@ -1445,15 +1551,6 @@ accounts_dialog_delete_event_cb (GtkWidget *widget,
   return TRUE;
 }
 
-static void
-accounts_dialog_destroy_cb (GtkObject *obj,
-    EmpathyAccountsDialog *dialog)
-{
-  DEBUG ("%p", obj);
-
-  g_object_unref (dialog);
-}
-
 static void
 accounts_dialog_set_selected_account (EmpathyAccountsDialog *dialog,
     TpAccount *account)
@@ -1515,6 +1612,8 @@ accounts_dialog_accounts_setup (EmpathyAccountsDialog *dialog)
 
       empathy_signal_connect_weak (l->data, "status-changed",
           G_CALLBACK (accounts_dialog_connection_changed_cb), G_OBJECT (dialog));
+      empathy_signal_connect_weak (l->data, "presence-changed",
+          G_CALLBACK (accounts_dialog_presence_changed_cb), G_OBJECT (dialog));
     }
   g_list_free (accounts);
 
@@ -1544,71 +1643,145 @@ accounts_dialog_manager_ready_cb (GObject *source_object,
   accounts_dialog_accounts_setup (user_data);
 }
 
+static void
+dialog_response_cb (GtkWidget *widget,
+    gint response_id,
+    gpointer user_data)
+{
+  EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (widget);
+
+  if (response_id == GTK_RESPONSE_HELP)
+    {
+      empathy_url_show (widget, "ghelp:empathy?accounts-window");
+    }
+  else if (response_id == GTK_RESPONSE_CLOSE ||
+      response_id == GTK_RESPONSE_DELETE_EVENT)
+    {
+      TpAccount *account = NULL;
+
+      if (accounts_dialog_has_pending_change (dialog, &account))
+        {
+          gchar *question_dialog_primary_text = get_dialog_primary_text (
+              account);
+
+          accounts_dialog_show_question_dialog (dialog,
+              question_dialog_primary_text,
+              _("You are about to close the window, which will discard\n"
+                  "your changes. Are you sure you want to proceed?"),
+              G_CALLBACK (accounts_dialog_close_response_cb),
+              widget,
+              GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
+              GTK_STOCK_DISCARD, GTK_RESPONSE_YES, NULL);
+
+          g_free (question_dialog_primary_text);
+        }
+      else
+        {
+          gtk_widget_destroy (widget);
+        }
+    }
+}
+
 static void
 accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
 {
+  GtkWidget *top_hbox;
   GtkBuilder                   *gui;
   gchar                        *filename;
   EmpathyAccountsDialogPriv    *priv = GET_PRIV (dialog);
-#ifdef HAVE_MOBLIN
-  GtkWidget                    *action_area;
-#endif
+  GtkWidget                    *content_area;
+  GtkWidget *action_area;
 
   filename = empathy_file_lookup ("empathy-accounts-dialog.ui", "src");
 
   gui = empathy_builder_get_file (filename,
-      "accounts_dialog", &priv->window,
+      "accounts_dialog_hbox", &top_hbox,
       "vbox_details", &priv->vbox_details,
       "frame_no_protocol", &priv->frame_no_protocol,
       "alignment_settings", &priv->alignment_settings,
+      "alignment_infobar", &priv->alignment_infobar,
       "treeview", &priv->treeview,
-      "frame_new_account", &priv->frame_new_account,
-      "hbox_type", &priv->hbox_type,
-      "button_create", &priv->button_create,
-      "button_back", &priv->button_back,
-      "radiobutton_reuse", &priv->radiobutton_reuse,
-      "radiobutton_register", &priv->radiobutton_register,
       "image_type", &priv->image_type,
       "label_name", &priv->label_name,
       "button_add", &priv->button_add,
       "button_import", &priv->button_import,
+      "hbox_protocol", &priv->hbox_protocol,
       NULL);
   g_free (filename);
 
   empathy_builder_connect (gui, dialog,
-      "accounts_dialog", "response", accounts_dialog_response_cb,
-      "accounts_dialog", "destroy", accounts_dialog_destroy_cb,
-      "accounts_dialog", "delete-event", accounts_dialog_delete_event_cb,
-      "button_create", "clicked", accounts_dialog_button_create_clicked_cb,
-      "button_back", "clicked", accounts_dialog_button_back_clicked_cb,
       "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
-      "button_help", "clicked", accounts_dialog_button_help_clicked_cb,
       "button_import", "clicked", accounts_dialog_button_import_clicked_cb,
       NULL);
 
+  content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+
+  gtk_container_add (GTK_CONTAINER (content_area), top_hbox);
+
   g_object_unref (gui);
 
-#ifdef HAVE_MOBLIN
-  action_area = gtk_dialog_get_action_area (GTK_DIALOG (priv->window));
-  gtk_widget_hide (action_area);
+  action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
 
-  /* Translators: this is used only when built on a moblin platform */
-  gtk_button_set_label (GTK_BUTTON (priv->button_create), _("_Next"));
-  gtk_button_set_use_underline (GTK_BUTTON (priv->button_create), TRUE);
+#ifdef HAVE_MOBLIN
+  gtk_widget_hide (action_area);
 #endif
 
   priv->combobox_protocol = empathy_protocol_chooser_new ();
-  gtk_box_pack_start (GTK_BOX (priv->hbox_type),
-      priv->combobox_protocol,
+  gtk_box_pack_start (GTK_BOX (priv->hbox_protocol), priv->combobox_protocol,
       TRUE, TRUE, 0);
-  gtk_widget_show (priv->combobox_protocol);
   g_signal_connect (priv->combobox_protocol, "changed",
       G_CALLBACK (accounts_dialog_protocol_changed_cb),
       dialog);
 
   if (priv->parent_window)
-    gtk_window_set_transient_for (GTK_WINDOW (priv->window),
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
         priv->parent_window);
+
+  /* set up spinner */
+  priv->throbber = ephy_spinner_new ();
+  ephy_spinner_set_size (EPHY_SPINNER (priv->throbber), GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+  priv->infobar = gtk_info_bar_new ();
+  gtk_container_add (GTK_CONTAINER (priv->alignment_infobar),
+      priv->infobar);
+  gtk_widget_show (priv->infobar);
+
+  priv->image_status = gtk_image_new_from_icon_name (
+            empathy_icon_name_for_presence (
+            TP_CONNECTION_PRESENCE_TYPE_OFFLINE), GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+  priv->label_status = gtk_label_new (NULL);
+
+  content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (priv->infobar));
+  gtk_box_pack_start (GTK_BOX (content_area), priv->throbber,
+      FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (content_area), priv->image_status,
+      FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (content_area), priv->label_status);
+
+  /* Tweak the dialog */
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Accounts"));
+  gtk_window_set_role (GTK_WINDOW (dialog), "accounts");
+
+  gtk_window_set_default_size (GTK_WINDOW (dialog), 640, -1);
+
+  gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
+
+  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
+  /* add dialog buttons */
+  gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area), GTK_BUTTONBOX_END);
+
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+      GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+      NULL);
+
+  g_signal_connect (dialog, "response",
+      G_CALLBACK (dialog_response_cb), dialog);
+
+  g_signal_connect (dialog, "delete-event",
+      G_CALLBACK (accounts_dialog_delete_event_cb), dialog);
 }
 
 static void
@@ -1617,6 +1790,11 @@ do_dispose (GObject *obj)
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (obj);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
+  if (priv->dispose_has_run)
+    return;
+
+  priv->dispose_has_run = TRUE;
+
   /* Disconnect signals */
   g_signal_handlers_disconnect_by_func (priv->account_manager,
       accounts_dialog_account_validity_changed_cb,
@@ -1666,7 +1844,6 @@ do_constructor (GType type,
   if (dialog_singleton)
     {
       retval = G_OBJECT (dialog_singleton);
-      g_object_ref (retval);
     }
   else
     {
@@ -1676,9 +1853,6 @@ do_constructor (GType type,
 
       dialog_singleton = EMPATHY_ACCOUNTS_DIALOG (retval);
       g_object_add_weak_pointer (retval, (gpointer) &dialog_singleton);
-      /* We add an extra reference that we'll release when the dialog is
-       * destroyed (accounts_dialog_destroy_cb) */
-      g_object_ref (retval);
     }
 
   return retval;
@@ -1749,7 +1923,7 @@ do_constructed (GObject *object)
 
           empathy_conf_set_bool (empathy_conf_get (),
               EMPATHY_PREFS_IMPORT_ASKED, TRUE);
-          import_dialog = empathy_import_dialog_new (GTK_WINDOW (priv->window),
+          import_dialog = empathy_import_dialog_new (GTK_WINDOW (dialog),
               FALSE);
           gtk_widget_show (import_dialog);
         }
@@ -1813,10 +1987,7 @@ empathy_accounts_dialog_show (GtkWindow *parent,
         priv->initial_selection = g_object_ref (selected_account);
     }
 
-  gtk_window_present (GTK_WINDOW (priv->window));
-  /* EmpathyAccountsDialog kepts a ref on itself until the dialog is
-   * destroyed so we can release the ref returned by the constructor now. */
-  g_object_unref (dialog);
+  gtk_window_present (GTK_WINDOW (dialog));
 
-  return priv->window;
+  return GTK_WIDGET (dialog);
 }
