@@ -16,6 +16,7 @@
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Authors: Jonny Lamb <jonny.lamb@collabora.co.uk>
+*           Cosimo Cecchi <cosimo.cecchi@collabora.co.uk>
 */
 
 #include "config.h"
@@ -30,6 +31,7 @@
 #include <libempathy/empathy-utils.h>
 
 #include <libempathy-gtk/empathy-account-chooser.h>
+#include <libempathy-gtk/empathy-geometry.h>
 
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/util.h>
@@ -57,6 +59,7 @@ enum
 {
   COL_CM_NAME = 0,
   COL_CM_UNIQUE_NAME,
+  COL_CM_GONE,
   NUM_COLS_CM
 };
 
@@ -79,6 +82,9 @@ typedef struct
   GtkToolItem *level_label;
   GtkWidget *level_filter;
 
+  /* Cache */
+  GHashTable *all_cms;
+
   /* TreeView */
   GtkListStore *store;
   GtkTreeModel *store_filter;
@@ -92,6 +98,7 @@ typedef struct
   TpProxy *proxy;
   TpProxySignalConnection *new_debug_message_signal;
   TpProxySignalConnection *name_owner_changed_signal;
+  gulong invalid_signal_id;
 
   /* Whether NewDebugMessage will be fired */
   gboolean paused;
@@ -109,22 +116,22 @@ log_level_to_string (guint level)
   switch (level)
     {
     case EMP_DEBUG_LEVEL_ERROR:
-      return _("Error");
+      return "Error";
       break;
     case EMP_DEBUG_LEVEL_CRITICAL:
-      return _("Critical");
+      return "Critical";
       break;
     case EMP_DEBUG_LEVEL_WARNING:
-      return _("Warning");
+      return "Warning";
       break;
     case EMP_DEBUG_LEVEL_MESSAGE:
-      return _("Message");
+      return "Message";
       break;
     case EMP_DEBUG_LEVEL_INFO:
-      return _("Info");
+      return "Info";
       break;
     case EMP_DEBUG_LEVEL_DEBUG:
-      return _("Debug");
+      return "Debug";
       break;
     default:
       g_assert_not_reached ();
@@ -132,8 +139,76 @@ log_level_to_string (guint level)
     }
 }
 
+typedef struct
+{
+  gdouble timestamp;
+  gchar *domain;
+  guint level;
+  gchar *message;
+} DebugMessage;
+
+static DebugMessage *
+debug_message_new (gdouble timestamp,
+    const gchar *domain,
+    guint level,
+    const gchar *message)
+{
+  DebugMessage *retval = g_slice_new0 (DebugMessage);
+
+  retval->timestamp = timestamp;
+  retval->domain = g_strdup (domain);
+  retval->level = level;
+  retval->message = g_strdup (message);
+
+  return retval;
+}
+
+static void
+debug_message_free (DebugMessage *dm)
+{
+  g_free (dm->domain);
+  g_free (dm->message);
+
+  g_slice_free (DebugMessage, dm);
+}
+
+static void
+debug_message_list_free (gpointer data)
+{
+  GList *list = data;
+
+  g_list_foreach (list, (GFunc) debug_message_free, NULL);
+  g_list_free (list);
+}
+
+static void
+debug_window_cache_new_message (EmpathyDebugWindow *debug_window,
+    gdouble timestamp,
+    const gchar *domain,
+    guint level,
+    const gchar *message)
+{
+  EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
+  GtkTreeIter iter;
+  GList *messages;
+  DebugMessage *dm;
+  char *name;
+
+  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter);
+
+  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
+      COL_CM_NAME, &name, -1);
+  messages = g_hash_table_lookup (priv->all_cms, name);
+
+  dm = debug_message_new (timestamp, domain, level, message);
+  messages = g_list_append (messages, dm);
+
+  g_hash_table_insert (priv->all_cms, name, messages);
+}
+
 static void
 debug_window_add_message (EmpathyDebugWindow *debug_window,
+    gboolean should_cache,
     gdouble timestamp,
     const gchar *domain_category,
     guint level,
@@ -144,6 +219,10 @@ debug_window_add_message (EmpathyDebugWindow *debug_window,
   GtkTreeIter iter;
   gchar *string;
 
+  if (should_cache)
+    debug_window_cache_new_message (debug_window, timestamp, domain_category,
+        level, message);
+
   if (g_strrstr (domain_category, "/"))
     {
       gchar **parts = g_strsplit (domain_category, "/", 2);
@@ -189,7 +268,7 @@ debug_window_new_debug_message_cb (TpProxy *proxy,
 {
   EmpathyDebugWindow *debug_window = (EmpathyDebugWindow *) user_data;
 
-  debug_window_add_message (debug_window, timestamp, domain, level,
+  debug_window_add_message (debug_window, TRUE, timestamp, domain, level,
       message);
 }
 
@@ -249,7 +328,10 @@ debug_window_get_messages_cb (TpProxy *proxy,
 {
   EmpathyDebugWindow *debug_window = (EmpathyDebugWindow *) user_data;
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
-  gint i;
+  GtkTreeIter iter;
+  gchar *name;
+  GList *old_messages;
+  guint i;
 
   if (error != NULL)
     {
@@ -260,11 +342,27 @@ debug_window_get_messages_cb (TpProxy *proxy,
 
   debug_window_set_toolbar_sensitivity (debug_window, TRUE);
 
+  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->cm_chooser), &iter);
+
+  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
+      COL_CM_NAME, &name, -1);
+  old_messages = g_hash_table_lookup (priv->all_cms, name);
+
+  /* we call get_messages either when a new CM is added or
+   * when a CM that we've already seen re-appears; in both cases
+   * we don't need our old cache anymore.
+   */
+  if (old_messages != NULL)
+    {
+      g_hash_table_remove (priv->all_cms, name);
+      debug_message_list_free (old_messages);
+    }
+
   for (i = 0; i < messages->len; i++)
     {
       GValueArray *values = g_ptr_array_index (messages, i);
 
-      debug_window_add_message (debug_window,
+      debug_window_add_message (debug_window, TRUE,
           g_value_get_double (g_value_array_get_nth (values, 0)),
           g_value_get_string (g_value_array_get_nth (values, 1)),
           g_value_get_uint (g_value_array_get_nth (values, 2)),
@@ -280,6 +378,43 @@ debug_window_get_messages_cb (TpProxy *proxy,
   debug_window_set_enabled (debug_window, !priv->paused);
 }
 
+static void
+debug_window_add_log_messages_from_cache (EmpathyDebugWindow *debug_window,
+    const gchar *name)
+{
+  GList *messages, *l;
+  DebugMessage *dm;
+  EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
+
+  DEBUG ("Adding logs from cache for CM %s", name);
+
+  messages = g_hash_table_lookup (priv->all_cms, name);
+
+  if (messages == NULL)
+    return;
+
+  for (l = messages; l != NULL; l = l->next)
+    {
+      dm = l->data;
+
+      debug_window_add_message (debug_window, FALSE, dm->timestamp,
+          dm->domain, dm->level, dm->message);
+    }
+}
+
+static void
+proxy_invalidated_cb (TpProxy *proxy,
+    guint domain,
+    gint code,
+    gchar *msg,
+    EmpathyDebugWindowPriv *self)
+{
+  EmpathyDebugWindowPriv *priv = GET_PRIV (self);
+
+  /* Proxy has been invalidated so we can't disconnect the signal any more */
+  priv->new_debug_message_signal = NULL;
+}
+
 static void
 debug_window_cm_chooser_changed_cb (GtkComboBox *cm_chooser,
     EmpathyDebugWindow *debug_window)
@@ -287,9 +422,10 @@ debug_window_cm_chooser_changed_cb (GtkComboBox *cm_chooser,
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
   TpDBusDaemon *dbus;
   GError *error = NULL;
-  gchar *bus_name;
+  gchar *bus_name, *name = NULL;
   TpProxy *proxy;
   GtkTreeIter iter;
+  gboolean cm_gone;
 
   if (!gtk_combo_box_get_active_iter (cm_chooser, &iter))
     {
@@ -302,6 +438,20 @@ debug_window_cm_chooser_changed_cb (GtkComboBox *cm_chooser,
       return;
     }
 
+  gtk_list_store_clear (priv->store);
+
+  gtk_tree_model_get (GTK_TREE_MODEL (priv->cms), &iter,
+      COL_CM_NAME, &name, COL_CM_GONE, &cm_gone, -1);
+
+  if (cm_gone)
+    {
+      debug_window_add_log_messages_from_cache (debug_window, name);
+      g_free (name);
+      return;
+    }
+
+  g_free (name);
+
   dbus = tp_dbus_daemon_dup (&error);
 
   if (error != NULL)
@@ -318,8 +468,6 @@ debug_window_cm_chooser_changed_cb (GtkComboBox *cm_chooser,
       NULL);
   g_free (bus_name);
 
-  gtk_list_store_clear (priv->store);
-
   /* Disable debug signalling */
   if (priv->proxy != NULL)
     debug_window_set_enabled (debug_window, FALSE);
@@ -332,7 +480,10 @@ debug_window_cm_chooser_changed_cb (GtkComboBox *cm_chooser,
     }
 
   if (priv->proxy != NULL)
-    g_object_unref (priv->proxy);
+    {
+      g_signal_handler_disconnect (priv->proxy, priv->invalid_signal_id);
+      g_object_unref (priv->proxy);
+    }
 
   priv->proxy = proxy;
 
@@ -341,13 +492,18 @@ debug_window_cm_chooser_changed_cb (GtkComboBox *cm_chooser,
   emp_cli_debug_call_get_messages (priv->proxy, -1,
       debug_window_get_messages_cb, debug_window, NULL, NULL);
 
+  priv->invalid_signal_id = g_signal_connect (proxy, "invalidated",
+      G_CALLBACK (proxy_invalidated_cb), debug_window);
+
   g_object_unref (dbus);
 }
 
 typedef struct
 {
-  const gchar *unique_name;
+  const gchar *name;
   gboolean found;
+  gboolean use_name;
+  GtkTreeIter **found_iter;
 } CmInModelForeachData;
 
 static gboolean
@@ -357,31 +513,41 @@ debug_window_cms_foreach (GtkTreeModel *model,
     gpointer user_data)
 {
   CmInModelForeachData *data = (CmInModelForeachData *) user_data;
-  gchar *unique_name;
+  gchar *store_name;
 
   gtk_tree_model_get (model, iter,
-      COL_CM_UNIQUE_NAME, &unique_name,
+      (data->use_name ? COL_CM_NAME : COL_CM_UNIQUE_NAME),
+      &store_name,
       -1);
 
-  if (!tp_strdiff (unique_name, data->unique_name))
-    data->found = TRUE;
+  if (!tp_strdiff (store_name, data->name))
+    {
+      data->found = TRUE;
 
-  g_free (unique_name);
+      if (data->found_iter != NULL)
+        *(data->found_iter) = gtk_tree_iter_copy (iter);
+    }
+
+  g_free (store_name);
 
   return data->found;
 }
 
 static gboolean
 debug_window_cm_is_in_model (EmpathyDebugWindow *debug_window,
-    const gchar *unique_name)
+    const gchar *name,
+    GtkTreeIter **iter,
+    gboolean use_name)
 {
   EmpathyDebugWindowPriv *priv = GET_PRIV (debug_window);
   CmInModelForeachData *data;
   gboolean found;
 
   data = g_slice_new0 (CmInModelForeachData);
-  data->unique_name = unique_name;
+  data->name = name;
   data->found = FALSE;
+  data->found_iter = iter;
+  data->use_name = use_name;
 
   gtk_tree_model_foreach (GTK_TREE_MODEL (priv->cms),
       debug_window_cms_foreach, data);
@@ -415,7 +581,7 @@ debug_window_get_name_owner_cb (TpDBusDaemon *proxy,
       goto OUT;
     }
 
-  if (!debug_window_cm_is_in_model (data->debug_window, out))
+  if (!debug_window_cm_is_in_model (data->debug_window, out, NULL, FALSE))
     {
       GtkTreeIter iter;
 
@@ -479,29 +645,6 @@ debug_window_list_connection_names_cb (const gchar * const *names,
   g_object_unref (dbus);
 }
 
-static gboolean
-debug_window_remove_cm_foreach (GtkTreeModel *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    gpointer user_data)
-{
-  const gchar *unique_name_to_remove = (const gchar *) user_data;
-  gchar *name;
-  gboolean found = FALSE;
-
-  gtk_tree_model_get (model, iter, COL_CM_UNIQUE_NAME, &name, -1);
-
-  if (!tp_strdiff (name, unique_name_to_remove))
-    {
-      found = TRUE;
-      gtk_list_store_remove (GTK_LIST_STORE (model), iter);
-    }
-
-  g_free (name);
-
-  return found;
-}
-
 #define CM_WELL_KNOWN_NAME_PREFIX \
     "org.freedesktop.Telepathy.ConnectionManager."
 
@@ -526,16 +669,41 @@ debug_window_name_owner_changed_cb (TpDBusDaemon *proxy,
        * just joined), we don't need to check whether the unique
        * name is in the CM model. Hooray.
        */
-      GtkTreeIter iter;
       const gchar *name = arg0 + strlen (CM_WELL_KNOWN_NAME_PREFIX);
 
-      DEBUG ("Adding new CM '%s' at %s.", name, arg2);
+      if (!g_hash_table_lookup (priv->all_cms, name))
+        {
+          GtkTreeIter iter;
+          DEBUG ("Adding new CM '%s' at %s.", name, arg2);
 
-      gtk_list_store_append (priv->cms, &iter);
-      gtk_list_store_set (priv->cms, &iter,
-          COL_CM_NAME, name,
-          COL_CM_UNIQUE_NAME, arg2,
-          -1);
+          gtk_list_store_append (priv->cms, &iter);
+          gtk_list_store_set (priv->cms, &iter,
+              COL_CM_NAME, name,
+              COL_CM_UNIQUE_NAME, arg2,
+              -1);
+        }
+      else
+        {
+          /* a CM with the same name is already in the hash table,
+           * update it and set it as re-enabled in the model.
+           */
+          GtkTreeIter *iter = NULL;
+
+          if (debug_window_cm_is_in_model (user_data, name, &iter, TRUE))
+            {
+              DEBUG ("Refreshing CM '%s' at '%s'.", name, arg2);
+
+              gtk_list_store_set (priv->cms, iter,
+                  COL_CM_NAME, name,
+                  COL_CM_UNIQUE_NAME, arg2,
+                  COL_CM_GONE, FALSE,
+                  -1);
+              gtk_tree_iter_free (iter);
+
+              debug_window_cm_chooser_changed_cb
+                (GTK_COMBO_BOX (priv->cm_chooser), user_data);
+            }
+        }
     }
   else if (!EMP_STR_EMPTY (arg1) && EMP_STR_EMPTY (arg2))
     {
@@ -544,17 +712,17 @@ debug_window_name_owner_changed_cb (TpDBusDaemon *proxy,
        * just died), we don't need to check that it was already
        * in the model.
        */
-      gchar *to_remove;
+      GtkTreeIter *iter = NULL;
 
-      /* Can't pass a const into a GtkTreeModelForeachFunc. */
-      to_remove = g_strdup (arg1);
+      DEBUG ("Setting CM disabled from %s.", arg1);
 
-      DEBUG ("Removing CM from %s.", to_remove);
-
-      gtk_tree_model_foreach (GTK_TREE_MODEL (priv->cms),
-          debug_window_remove_cm_foreach, to_remove);
-
-      g_free (to_remove);
+      /* set the CM as disabled in the model */
+      if (debug_window_cm_is_in_model (user_data, arg1, &iter, FALSE))
+	{
+	  gtk_list_store_set (priv->cms,
+              iter, COL_CM_GONE, TRUE, -1);
+          gtk_tree_iter_free (iter);
+        }
     }
 }
 
@@ -586,6 +754,13 @@ debug_window_fill_cm_chooser (EmpathyDebugWindow *debug_window)
   tp_list_connection_names (priv->dbus, debug_window_list_connection_names_cb,
       debug_window, NULL, NULL);
 
+  /* add Mission Control */
+  gtk_list_store_append (priv->cms, &iter);
+  gtk_list_store_set (priv->cms, &iter,
+      COL_CM_NAME, "misson-control",
+      COL_CM_UNIQUE_NAME, "org.freedesktop.Telepathy.MissionControl5",
+      -1);
+
   priv->name_owner_changed_signal =
       tp_cli_dbus_daemon_connect_to_name_owner_changed (priv->dbus,
       debug_window_name_owner_changed_cb, debug_window, NULL, NULL, NULL);
@@ -741,6 +916,48 @@ debug_window_button_press_event_cb (GtkTreeView *view,
   return FALSE;
 }
 
+static gchar *
+debug_window_format_timestamp (gdouble timestamp)
+{
+  struct tm *tstruct;
+  char time_str[32];
+  gint ms;
+  time_t sec;
+  gchar *text;
+
+  ms = (int) ((timestamp - (int) timestamp)*1e6);
+  sec = (long) timestamp;
+  tstruct = localtime ((time_t *) &sec);
+  if (!strftime (time_str, sizeof (time_str), "%x %T", tstruct))
+    {
+      DEBUG ("Failed to format timestamp: %e", timestamp);
+      time_str[0] = '\0';
+    }
+
+  text = g_strdup_printf ("%s.%d", time_str, ms);
+
+  return text;
+}
+
+static void
+debug_window_time_formatter (GtkTreeViewColumn *tree_column,
+    GtkCellRenderer *cell,
+    GtkTreeModel *tree_model,
+    GtkTreeIter *iter,
+    gpointer data)
+{
+  gdouble timestamp;
+  gchar *time_str;
+
+  gtk_tree_model_get (tree_model, iter, COL_DEBUG_TIMESTAMP, &timestamp, -1);
+
+  time_str = debug_window_format_timestamp (timestamp);
+
+  g_object_set (G_OBJECT (cell), "text", time_str, NULL);
+
+  g_free (time_str);
+}
+
 static gboolean
 debug_window_store_filter_foreach (GtkTreeModel *model,
     GtkTreePath *path,
@@ -750,7 +967,7 @@ debug_window_store_filter_foreach (GtkTreeModel *model,
   GFileOutputStream *output_stream = (GFileOutputStream *) user_data;
   gchar *domain, *category, *message, *level_str, *level_upper;
   gdouble timestamp;
-  gchar *line;
+  gchar *line, *time_str;
   GError *error = NULL;
   gboolean out = FALSE;
 
@@ -764,9 +981,13 @@ debug_window_store_filter_foreach (GtkTreeModel *model,
 
   level_upper = g_ascii_strup (level_str, -1);
 
-  line = g_strdup_printf ("%s%s%s-%s: %e: %s\n",
+  time_str = debug_window_format_timestamp (timestamp);
+
+  line = g_strdup_printf ("%s%s%s-%s: %s: %s\n",
       domain, EMP_STR_EMPTY (category) ? "" : "/",
-      category, level_upper, timestamp, message);
+      category, level_upper, time_str, message);
+
+  g_free (time_str);
 
   g_output_stream_write (G_OUTPUT_STREAM (output_stream), line,
       strlen (line), NULL, &error);
@@ -866,7 +1087,7 @@ debug_window_copy_model_foreach (GtkTreeModel *model,
   gchar *tmp;
   gchar *domain, *category, *message, *level_str, *level_upper;
   gdouble timestamp;
-  gchar *line;
+  gchar *line, *time_str;
 
   gtk_tree_model_get (model, iter,
       COL_DEBUG_TIMESTAMP, &timestamp,
@@ -878,9 +1099,13 @@ debug_window_copy_model_foreach (GtkTreeModel *model,
 
   level_upper = g_ascii_strup (level_str, -1);
 
-  line = g_strdup_printf ("%s%s%s-%s: %e: %s\n",
+  time_str = debug_window_format_timestamp (timestamp);
+
+  line = g_strdup_printf ("%s%s%s-%s: %s: %s\n",
       domain, EMP_STR_EMPTY (category) ? "" : "/",
-      category, level_upper, timestamp, message);
+      category, level_upper, time_str, message);
+
+  g_free (time_str);
 
   tmp = g_strconcat (*text, line, NULL);
 
@@ -927,7 +1152,8 @@ debug_window_key_press_event_cb (GtkWidget *widget,
     GdkEventKey *event,
     gpointer user_data)
 {
-  if (event->state & GDK_CONTROL_MASK && event->keyval == GDK_w)
+  if ((event->state & GDK_CONTROL_MASK && event->keyval == GDK_w)
+      || event->keyval == GDK_Escape)
     {
       gtk_widget_destroy (widget);
       return TRUE;
@@ -958,6 +1184,7 @@ debug_window_constructor (GType type,
 
   gtk_window_set_title (GTK_WINDOW (object), _("Debug Window"));
   gtk_window_set_default_size (GTK_WINDOW (object), 800, 400);
+  empathy_geometry_bind (GTK_WINDOW (object), "debug-window");
 
   g_signal_connect (object, "key-press-event",
       G_CALLBACK (debug_window_key_press_event_cb), NULL);
@@ -977,7 +1204,8 @@ debug_window_constructor (GType type,
 
   /* CM */
   priv->cm_chooser = gtk_combo_box_new_text ();
-  priv->cms = gtk_list_store_new (NUM_COLS_CM, G_TYPE_STRING, G_TYPE_STRING);
+  priv->cms = gtk_list_store_new (NUM_COLS_CM, G_TYPE_STRING, G_TYPE_STRING,
+      G_TYPE_BOOLEAN);
   gtk_combo_box_set_model (GTK_COMBO_BOX (priv->cm_chooser),
       GTK_TREE_MODEL (priv->cms));
   gtk_widget_show (priv->cm_chooser);
@@ -1114,8 +1342,9 @@ debug_window_constructor (GType type,
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "yalign", 0, NULL);
 
-  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->view),
-      -1, _("Time"), renderer, "text", COL_DEBUG_TIMESTAMP, NULL);
+  gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (priv->view),
+      -1, _("Time"), renderer,
+      (GtkTreeCellDataFunc) debug_window_time_formatter, NULL, NULL);
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->view),
       -1, _("Domain"), renderer, "text", COL_DEBUG_DOMAIN, NULL);
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->view),
@@ -1156,7 +1385,8 @@ debug_window_constructor (GType type,
           _("The selected connection manager does not support the remote "
               "debugging extension.")));
   gtk_widget_show (priv->not_supported_label);
-  gtk_box_pack_start (GTK_BOX (vbox), priv->not_supported_label, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), priv->not_supported_label,
+      TRUE, TRUE, 0);
 
   priv->view_visible = FALSE;
 
@@ -1177,6 +1407,8 @@ empathy_debug_window_init (EmpathyDebugWindow *empathy_debug_window)
   empathy_debug_window->priv = priv;
 
   priv->dispose_run = FALSE;
+  priv->all_cms = g_hash_table_new_full (g_str_hash, g_str_equal,
+      g_free, NULL);
 }
 
 static void
@@ -1207,6 +1439,27 @@ debug_window_get_property (GObject *object,
     }
 }
 
+static void
+debug_window_finalize (GObject *object)
+{
+  EmpathyDebugWindowPriv *priv = GET_PRIV (object);
+  GHashTableIter iter;
+  char *key;
+  GList *values;
+
+  g_hash_table_iter_init (&iter, priv->all_cms);
+
+  while (g_hash_table_iter_next (&iter, (gpointer *) &key,
+          (gpointer *) &values))
+    {
+      debug_message_list_free (values);
+    }
+
+  g_hash_table_destroy (priv->all_cms);
+
+  (G_OBJECT_CLASS (empathy_debug_window_parent_class)->finalize) (object);
+}
+
 static void
 debug_window_dispose (GObject *object)
 {
@@ -1227,6 +1480,7 @@ debug_window_dispose (GObject *object)
   if (priv->proxy != NULL)
     {
       debug_window_set_enabled (EMPATHY_DEBUG_WINDOW (object), FALSE);
+      g_signal_handler_disconnect (priv->proxy, priv->invalid_signal_id);
       g_object_unref (priv->proxy);
     }
 
@@ -1248,6 +1502,7 @@ empathy_debug_window_class_init (EmpathyDebugWindowClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   object_class->constructor = debug_window_constructor;
   object_class->dispose = debug_window_dispose;
+  object_class->finalize = debug_window_finalize;
   object_class->set_property = debug_window_set_property;
   object_class->get_property = debug_window_get_property;
   g_type_class_add_private (klass, sizeof (EmpathyDebugWindowPriv));
@@ -1258,7 +1513,7 @@ empathy_debug_window_class_init (EmpathyDebugWindowClass *klass)
 GtkWidget *
 empathy_debug_window_new (GtkWindow *parent)
 {
-  g_return_val_if_fail (GTK_IS_WINDOW (parent), NULL);
+  g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), NULL);
 
   return GTK_WIDGET (g_object_new (EMPATHY_TYPE_DEBUG_WINDOW,
       "transient-for", parent, NULL));
