@@ -96,6 +96,7 @@ struct _EventPriv {
   EventFunc func;
   gboolean inhibit;
   gpointer user_data;
+  guint autoremove_timeout_id;
 };
 
 enum {
@@ -160,6 +161,9 @@ event_free (EventPriv *event)
   g_free (event->public.header);
   g_free (event->public.message);
 
+  if (event->autoremove_timeout_id != 0)
+    g_source_remove (event->autoremove_timeout_id);
+
   if (event->public.contact)
     {
       g_object_unref (event->public.contact);
@@ -174,6 +178,7 @@ event_remove (EventPriv *event)
   EmpathyEventManagerPriv *priv = GET_PRIV (event->manager);
 
   DEBUG ("Removing event %p", event);
+
   priv->events = g_slist_remove (priv->events, event);
   g_signal_emit (event->manager, signals[EVENT_REMOVED], 0, event);
   event_free (event);
@@ -182,6 +187,7 @@ event_remove (EventPriv *event)
 static gboolean
 autoremove_event_timeout_cb (EventPriv *event)
 {
+  event->autoremove_timeout_id = 0;
   event_remove (event);
   return FALSE;
 }
@@ -219,8 +225,9 @@ event_manager_add (EmpathyEventManager *manager,
 
   if (!event->public.must_ack)
     {
-      g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
-        (GSourceFunc) autoremove_event_timeout_cb, event);
+      event->autoremove_timeout_id = g_timeout_add_seconds (
+          NOTIFICATION_TIMEOUT, (GSourceFunc) autoremove_event_timeout_cb,
+          event);
     }
 }
 
@@ -319,6 +326,8 @@ event_channel_process_voip_func (EventPriv *event)
   GtkWidget *dialog;
   GtkWidget *button;
   GtkWidget *image;
+  EmpathyTpCall *call;
+  gboolean video;
 
   if (event->approval->dialog != NULL)
     {
@@ -326,13 +335,36 @@ event_channel_process_voip_func (EventPriv *event)
       return;
     }
 
+  call = EMPATHY_TP_CALL (empathy_dispatch_operation_get_channel_wrapper (
+        event->approval->operation));
+
+  video = empathy_tp_call_has_initial_video (call);
+
   dialog = gtk_message_dialog_new (NULL, 0,
-      GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Incoming call"));
+      GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+      video ? _("Incoming video call"): _("Incoming call"));
+
   gtk_message_dialog_format_secondary_text (
-    GTK_MESSAGE_DIALOG (dialog),
-      _("%s is calling you, do you want to answer?"),
+    GTK_MESSAGE_DIALOG (dialog), video ?
+      _("%s is video calling you. Do you want to answer?"):
+      _("%s is calling you. Do you want to answer?"),
       empathy_contact_get_name (event->approval->contact));
 
+  /* Set image of the dialog */
+  if (video)
+    {
+      image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VIDEO_CALL,
+          GTK_ICON_SIZE_DIALOG);
+    }
+  else
+    {
+      image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_VOIP,
+          GTK_ICON_SIZE_DIALOG);
+    }
+
+  gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
+  gtk_widget_show (image);
+
   gtk_dialog_set_default_response (GTK_DIALOG (dialog),
       GTK_RESPONSE_OK);
 
@@ -451,12 +483,22 @@ event_manager_media_channel_got_contact (EventManagerApproval *approval)
 {
   EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
   gchar *header;
+  EmpathyTpCall *call;
+  gboolean video;
 
-  header = g_strdup_printf (_("Incoming call from %s"),
+  call = EMPATHY_TP_CALL (empathy_dispatch_operation_get_channel_wrapper (
+        approval->operation));
+
+  video = empathy_tp_call_has_initial_video (call);
+
+  header = g_strdup_printf (
+    video ? _("Incoming video call from %s") :_("Incoming call from %s"),
     empathy_contact_get_name (approval->contact));
 
   event_manager_add (approval->manager, approval->contact,
-      EMPATHY_EVENT_TYPE_VOIP, EMPATHY_IMAGE_VOIP, header, NULL, approval,
+      EMPATHY_EVENT_TYPE_VOIP,
+      video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
+      header, NULL, approval,
       event_channel_process_voip_func, NULL);
 
   g_free (header);
