@@ -48,6 +48,11 @@
 
 typedef struct {
 	EmpathyTpRoomlist *room_list;
+	/* Currently selected account */
+	TpAccount         *account;
+	/* Signal id of the "status-changed" signal connected on the currently
+	 * selected account */
+	gulong             status_changed_id;
 
 	GtkWidget         *window;
 	GtkWidget         *vbox_widgets;
@@ -239,6 +244,11 @@ new_chatroom_dialog_destroy_cb (GtkWidget               *widget,
 	}
   	g_object_unref (dialog->model);
 
+	if (dialog->account != NULL) {
+		g_signal_handler_disconnect (dialog->account, dialog->status_changed_id);
+		g_object_unref (dialog->account);
+	}
+
 	g_free (dialog);
 }
 
@@ -350,21 +360,42 @@ new_chatroom_dialog_model_add_columns (EmpathyNewChatroomDialog *dialog)
 	gtk_tree_view_append_column (view, column);
 }
 
+static void
+update_join_button_sensitivity (EmpathyNewChatroomDialog *dialog)
+{
+	const gchar           *room;
+	gboolean               sensitive = FALSE;
+
+
+	room = gtk_entry_get_text (GTK_ENTRY (dialog->entry_room));
+	if (EMP_STR_EMPTY (room))
+		goto out;
+
+	if (dialog->account == NULL)
+		goto out;
+
+	if (tp_account_get_connection_status (dialog->account, NULL) !=
+		      TP_CONNECTION_STATUS_CONNECTED)
+		goto out;
+
+	sensitive = TRUE;
+
+out:
+	gtk_widget_set_sensitive (dialog->button_join, sensitive);
+}
+
 static void
 new_chatroom_dialog_update_widgets (EmpathyNewChatroomDialog *dialog)
 {
 	EmpathyAccountChooser *account_chooser;
-	TpAccount             *account;
 	const gchar           *protocol;
-	const gchar           *room;
 
 	account_chooser = EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser);
-	account = empathy_account_chooser_dup_account (account_chooser);
 
-	if (account == NULL)
+	if (dialog->account == NULL)
 		return;
 
-	protocol = tp_account_get_protocol (account);
+	protocol = tp_account_get_protocol (dialog->account);
 
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_server), "");
 
@@ -382,13 +413,22 @@ new_chatroom_dialog_update_widgets (EmpathyNewChatroomDialog *dialog)
 		gtk_widget_set_sensitive (dialog->entry_server, TRUE);
 	}
 
-	room = gtk_entry_get_text (GTK_ENTRY (dialog->entry_room));
-	gtk_widget_set_sensitive (dialog->button_join, !EMP_STR_EMPTY (room));
+	update_join_button_sensitivity (dialog);
 
 	/* Final set up of the dialog */
 	gtk_widget_grab_focus (dialog->entry_room);
+}
 
-	g_object_unref (account);
+static void
+account_status_changed_cb (TpAccount *account,
+			    guint old_status,
+			    guint new_status,
+			    guint reason,
+			    gchar *dbus_error_name,
+			    GHashTable *details,
+			    EmpathyNewChatroomDialog *self)
+{
+	update_join_button_sensitivity (self);
 }
 
 static void
@@ -396,7 +436,6 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 					EmpathyNewChatroomDialog *dialog)
 {
 	EmpathyAccountChooser *account_chooser;
-	TpAccount             *account;
 	gboolean               listing = FALSE;
 	gboolean               expanded = FALSE;
 
@@ -408,12 +447,20 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 	ephy_spinner_stop (EPHY_SPINNER (dialog->throbber));
 	new_chatroom_dialog_model_clear (dialog);
 
+	if (dialog->account != NULL) {
+		g_signal_handler_disconnect (dialog->account, dialog->status_changed_id);
+		g_object_unref (dialog->account);
+	}
+
 	account_chooser = EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser);
-	account = empathy_account_chooser_dup_account (account_chooser);
-	if (account == NULL)
+	dialog->account = empathy_account_chooser_dup_account (account_chooser);
+	if (dialog->account == NULL)
 		goto out;
 
-	dialog->room_list = empathy_tp_roomlist_new (account);
+	dialog->status_changed_id = g_signal_connect (dialog->account,
+		      "status-changed", G_CALLBACK (account_status_changed_cb), dialog);
+
+	dialog->room_list = empathy_tp_roomlist_new (dialog->account);
 
 	if (dialog->room_list) {
 		g_signal_connect (dialog->room_list, "destroy",
@@ -445,8 +492,6 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 		}
 	}
 
-	g_object_unref (account);
-
 out:
 	new_chatroom_dialog_update_widgets (dialog);
 }
@@ -489,9 +534,9 @@ new_chatroom_dialog_new_room_cb (EmpathyTpRoomlist        *room_list,
 	selection = gtk_tree_view_get_selection (view);
 	store = GTK_LIST_STORE (dialog->model);
 	members = g_strdup_printf ("%d", empathy_chatroom_get_members_count (chatroom));
-	tooltip = g_strdup_printf (C_("Room/Join's roomlist tooltip. Parameters"
-		"are a channel name, yes/no, yes/no and a number.",
-		"<b>%s</b>\nInvite required: %s\nPassword required: %s\nMembers: %s"),
+	/* Translators: Room/Join's roomlist tooltip. Parameters are a channel name,
+	yes/no, yes/no and a number. */
+	tooltip = g_strdup_printf (_("<b>%s</b>\nInvite required: %s\nPassword required: %s\nMembers: %s"),
 		empathy_chatroom_get_name (chatroom),
 		empathy_chatroom_get_invite_only (chatroom) ? _("Yes") : _("No"),
 		empathy_chatroom_get_need_password (chatroom) ? _("Yes") : _("No"),
@@ -629,10 +674,8 @@ new_chatroom_dialog_entry_changed_cb (GtkWidget                *entry,
 				      EmpathyNewChatroomDialog *dialog)
 {
 	if (entry == dialog->entry_room) {
-		const gchar *room;
+		update_join_button_sensitivity (dialog);
 
-		room = gtk_entry_get_text (GTK_ENTRY (dialog->entry_room));
-		gtk_widget_set_sensitive (dialog->button_join, !EMP_STR_EMPTY (room));
 		/* FIXME: Select the room in the list */
 	}
 }
